from langchain.document_loaders import UnstructuredMarkdownLoader
from langchain.schema import Document
from langchain.embeddings import OpenAIEmbeddings
from langchain.vectorstores import Weaviate
from .documentTypes import DocumentType
from config import config
import logging
import weaviate
from typing import Iterable, List
from langchain.text_splitter import NLTKTextSplitter
from internal.weaviate.weaviate import WeaviateClient

# Globals
# =======
# The text splitter

textSplitter = NLTKTextSplitter(chunk_size=config["app.document.split_chunk_size"],
                 chunk_overlap=config["app.document.split_chunk_overlap"])
"""
The text splitter client
"""

db = None

def loadDocumentsIntoWeaviate(documentType: DocumentType,
                              documents: dict) -> None:
    """
    Loads a document into Weaviate.

    Expects documents to be of formate {name -> path}
    """
    texts = []

    for name, path in documents.items():
        documents = None

        if documentType == DocumentType.markdown:
            loader = UnstructuredMarkdownLoader(path)
            documents = loader.load()
        else:
            raise Exception("Document type not supported.")

        # Split up the document
        newDocuments = splitDocuments(documents)
        for doc in newDocuments:
            doc.metadata["document_name"] = name

        texts.extend(newDocuments)

    db = getDatabase()

    logging.info(f"Parsed {len(documents)} documents")

    db.addDocuments(documents=texts)

    logging.info(f"Loaded document {len(documents)} into Weaviate")

def loadDocumentIntoWeaviate(documentType: DocumentType,
                             document_name : str,
                             path: str) -> None:
    """
    Loads a document into Weaviate.
    """
    loadDocumentsIntoWeaviate(documentType, {document_name: path})

def findSimilarDocuments(query: str, limit : int = 10) -> List[Document]:
    """
    Finds similar documents to the query
    """
    db = getDatabase()
    return db.similaritySearch(query=query, k=limit)

def splitDocuments(document: Iterable[Document]) -> List[Document]:
    """
    Splits a document into multiple documents
    """
    if document is None:
        raise Exception("Document is None")

    global textSplitter

    return textSplitter.split_documents(document)

def getDatabase() -> WeaviateClient:
    """
    Get a weaviate client instance
    """
    global db

    if db is None:
        logging.debug(config["app.weaviate.url"])
        client = weaviate.Client(config["app.weaviate.url"])
        db = WeaviateClient(client,
                            text_key = config["app.weaviate.text-field"],
                            index_name = config["app.weaviate.index-name"])

    return db
